package com.dxhy.core.service.frJob;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dxhy.core.dao.frTask.RecordToQueryDao;
import com.dxhy.common.enums.KjkmTaxEnmu;
import com.dxhy.core.entity.frJob.FpTaxAmount;
import com.dxhy.core.entity.frJob.VoucherRecordQuery;
import com.dxhy.core.entity.frJob.VoucherRecordQueryDetail;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service("recordToQueryService")
public class RecordToQueryServiceImpl extends ServiceImpl<RecordToQueryDao, VoucherRecordQuery> implements RecordToQueryService {

    @Resource
    private RecordToQueryDao recordToQueryDao;

    @Resource
    private QueryDetailDataService queryDetailDataService;


    private  final String JXZZS = "22210101";

    private  final String XXZZS = "22210102";

    private  final String ZYYWSR = "6001";

    private  final String JXSZC = "2221010900";



    @Override
    public void getDataForFr() {

        //开始插入主数据信息
        try {
            boolean flag1 = this.insertJxDataToQuery();
            if (flag1){
                queryDetailDataService.insertJxDataToQueryDetail();
            }
        }catch (Exception e) {
            log.error("帆软进项增值税数据获取异常",e);
        }

        try {
            boolean flag2 = this.insertXxDataToQuery();
            if (flag2){
                queryDetailDataService.insertXxDataToQueryDetail();
            }
        }catch (Exception e) {
            log.error("帆软销项增值税数据获取异常",e);
        }

        try {
            boolean flag3 = this.insertZyDataToQuery();
            if (flag3){
                queryDetailDataService.insertZyDataToQueryDetail();
            }
        }catch (Exception e) {
            log.error("帆软主营业务收入数据获取异常",e);
        }
        this.insertOutDataToQuery();

        //更新发票认证状态
        List<String> uuids = queryDetailDataService.getRzztDetail();

        if (uuids != null && uuids.size() > 0){
            if (uuids.size() > 5000){

                int count = uuids.size()%5000==0?uuids.size()/5000:uuids.size()/5000+1;
                for (int i = 0; i < count; i++) {
                    int start = i*5000;
                    int end = (i+1)*5000;
                    if (i == count-1){
                        end = uuids.size();
                    }
                    List<String> subList = uuids.subList(start,end);
                    queryDetailDataService.updateRzzt(subList);
                }
            }else {
                List<String> rzztList = recordToQueryDao.getRzzt(uuids);

                if (rzztList != null && rzztList.size() > 0){
                    queryDetailDataService.updateRzzt(rzztList);
                }
            }
        }



    }

    public boolean insertJxDataToQuery() {

        boolean flag = false;
        //1.获取时间
        String[] time = getTime(JXZZS);

        //2.查询数据
        List<VoucherRecordQuery> voucherRecordQuerys =
                recordToQueryDao.getDataFromRecordItems(time[0], time[1],JXZZS);

        if (voucherRecordQuerys != null && voucherRecordQuerys.size() > 0){
            //3.查询税率
            Map<String, String> taxMap = KjkmTaxEnmu.getTaxMap();
            //4.清洗税率
            voucherRecordQuerys.forEach(v->v.setSl(taxMap.get(v.getKjkmbh())));

            //5.报账单号去重
            List<String> bzdhs
                    = voucherRecordQuerys.stream().map(VoucherRecordQuery::getBzdh)
                    .distinct().collect(Collectors.toList());

            //6.根据报账单号获取发票税额
            List<FpTaxAmount> taxAmounts = recordToQueryDao.getTaxAmount(bzdhs);
            if (taxAmounts != null && taxAmounts.size()>0){
                Map<String, String> amountMap = taxAmounts.stream().collect(Collectors.toMap(FpTaxAmount::getKey, FpTaxAmount::getFpse));
                //7.添加税率
                voucherRecordQuerys.forEach(v->v.setFpse(amountMap.get(v.getSl()+v.getBzdh())==null?"0":amountMap.get(v.getSl()+v.getBzdh())));
            }
            //8.保存或更新

            if (voucherRecordQuerys.size()>5000){
                int count = voucherRecordQuerys.size()%5000==0?voucherRecordQuerys.size()/5000:voucherRecordQuerys.size()/5000+1;
                for (int i = 0; i < count; i++) {
                    int start = i*5000;
                    int end = (i+1)*5000;
                    if (i == count-1){
                        end = voucherRecordQuerys.size();
                    }
                    List<VoucherRecordQuery> subList = voucherRecordQuerys.subList(start,end);
                    recordToQueryDao.insertOrUpdate(subList);
                }
            }else {
                recordToQueryDao.insertOrUpdate(voucherRecordQuerys);
            }
            flag = true;
        }
        return flag;
    }


    public boolean insertXxDataToQuery() {
        boolean flag = false;
        //1.获取时间
        String[] time = getTime(XXZZS);

        //2.查询数据
        List<VoucherRecordQuery> voucherRecordQuerys =
                recordToQueryDao.getDataXFromRecordItems(time[0],time[1],XXZZS);

        if (voucherRecordQuerys != null && voucherRecordQuerys.size() > 0) {
            //3.查询税率
            Map<String, String> taxMap = KjkmTaxEnmu.getTaxMap();
            //4.清洗税率
            voucherRecordQuerys.forEach(v -> {
                v.setSl(taxMap.get(v.getKjkmbh()));
            });

            //5.报账单号去重
            List<String> bzdhs
                    = voucherRecordQuerys.stream().map(VoucherRecordQuery::getBzdh)
                    .distinct().collect(Collectors.toList());

            //6.根据报账单号获取发票税额
            List<FpTaxAmount> taxAmounts = recordToQueryDao.getXTaxAmount(bzdhs);
            if (taxAmounts != null && taxAmounts.size() > 0) {
                Map<String, String> amountMap = taxAmounts.stream().collect(Collectors.toMap(FpTaxAmount::getKey, FpTaxAmount::getFpse));

                //7.添加税率
                voucherRecordQuerys.forEach(v -> v.setFpse(amountMap.get(v.getSl() + v.getBzdh()) == null ? "0" : amountMap.get(v.getSl() + v.getBzdh())));
            }

            //8.保存或更新

            if (voucherRecordQuerys.size()>5000){
                int count = voucherRecordQuerys.size()%5000==0?voucherRecordQuerys.size()/5000:voucherRecordQuerys.size()/5000+1;
                for (int i = 0; i < count; i++) {
                    int start = i*5000;
                    int end = (i+1)*5000;
                    if (i == count-1){
                        end = voucherRecordQuerys.size();
                    }
                    List<VoucherRecordQuery> subList = voucherRecordQuerys.subList(start,end);
                    recordToQueryDao.insertOrUpdate(subList);
                }
            }else {
                recordToQueryDao.insertOrUpdate(voucherRecordQuerys);
            }
            flag = true;
        }
        return flag;
    }


    public boolean insertZyDataToQuery() {
        boolean flag = false;
        //1.获取时间
        String[] time = getTime(ZYYWSR);

        //2.查询数据
        List<VoucherRecordQuery> voucherRecordQuerys =
                recordToQueryDao.getDataZFromRecordItems(time[0],time[1],ZYYWSR);

        if (voucherRecordQuerys != null && voucherRecordQuerys.size() > 0) {

            //3.报账单号去重
            List<String> bzdhs
                    = voucherRecordQuerys.stream().map(VoucherRecordQuery::getBzdh)
                    .distinct().collect(Collectors.toList());

            //4.根据报账单号获取发票税额
            List<FpTaxAmount> taxAmounts = recordToQueryDao.getZTaxAmount(bzdhs);
            if (taxAmounts != null && taxAmounts.size() > 0) {
                Map<String, String> amountMap = taxAmounts.stream().collect(Collectors.toMap(FpTaxAmount::getKey, FpTaxAmount::getFpje));

                //5.添加税率
                voucherRecordQuerys.forEach(v -> v.setFpse(amountMap.get(v.getSl() + v.getBzdh()) == null ? "0" : amountMap.get(v.getSl() + v.getBzdh())));
            }

            //6.保存或更新
            if (voucherRecordQuerys.size()>5000){
                int count = voucherRecordQuerys.size()%5000==0?voucherRecordQuerys.size()/5000:voucherRecordQuerys.size()/5000+1;
                for (int i = 0; i < count; i++) {
                    int start = i*5000;
                    int end = (i+1)*5000;
                    if (i == count-1){
                        end = voucherRecordQuerys.size();
                    }
                    List<VoucherRecordQuery> subList = voucherRecordQuerys.subList(start,end);
                    recordToQueryDao.insertOrUpdate(subList);
                }
            }else {
                recordToQueryDao.insertOrUpdate(voucherRecordQuerys);
            }
            flag = true;
        }
        return flag;
    }


    public void insertOutDataToQuery() {
        //获取时间
        String[] time = this.getTime(JXSZC);
        //查询数据
        List<VoucherRecordQuery> dataFromRecordItems = recordToQueryDao.getDataFromRecordItems(time[0], time[1], JXSZC);

        if (dataFromRecordItems != null && dataFromRecordItems.size() != 0){

            if (dataFromRecordItems.size()>5000){
                int count = dataFromRecordItems.size()%5000==0?dataFromRecordItems.size()/5000:dataFromRecordItems.size()/5000+1;
                for (int i = 0; i < count; i++) {
                    int start = i*5000;
                    int end = (i+1)*5000;
                    if (i == count-1){
                        end = dataFromRecordItems.size();
                    }
                    List<VoucherRecordQuery> subList = dataFromRecordItems.subList(start,end);
                    recordToQueryDao.insertOrUpdate(subList);
                }
            }else {
                recordToQueryDao.insertOrUpdate(dataFromRecordItems);
            }

        }

    }

    String[] getTime(String km){
        //1.获取增量数据对应时间
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        Date beginTime = recordToQueryDao.getBeginTime(km);

        //第一次数据为空
        if (beginTime == null){
            beginTime = recordToQueryDao.getBeginTimefromRecord();
        }
        Calendar cal = Calendar.getInstance();

        cal.setTime(beginTime);
        cal.add(Calendar.HOUR_OF_DAY,-1);

        String start = sdf.format(cal.getTime());
        //结束日期设置为明天
        cal.setTime(new Date());
        cal.add(Calendar.DAY_OF_MONTH,1);
        String end = sdf.format(cal.getTime());

        String[] time = {start,end};
        return time;
    }

}
